@@ -1,6 +1,6 @@
 use std::error::Error as StdError;
-use std::io::Error as IOError;
 use std::fmt;
+use std::io::Error as IOError;
 
 use protobuf::error::ProtobufError;
 use tokio_dl_stream_to_disk::error::Error as TDSTDError;
@@ -11,6 +11,7 @@ pub enum ErrorKind {
     FileExists,
     DirectoryMissing,
     InvalidApp,
+    SecurityCheck,
     IO(IOError),
     Str(String),
     Protobuf(ProtobufError),
@@ -24,9 +25,7 @@ pub struct Error {
 
 impl Error {
     pub fn new(k: ErrorKind) -> Error {
-        Error {
-            kind: k,
-        }
+        Error { kind: k }
     }
 
     pub fn kind(&self) -> &ErrorKind {
@@ -38,7 +37,7 @@ impl From<IOError> for Error {
     fn from(err: IOError) -> Error {
         Error {
             kind: ErrorKind::IO(err),
-	}
+        }
     }
 }
 
@@ -46,26 +45,30 @@ impl From<Box<dyn StdError>> for Error {
     fn from(err: Box<dyn StdError>) -> Error {
         Error {
             kind: ErrorKind::Other(err),
-	}
+        }
     }
 }
 
 impl From<TDSTDError> for Error {
     fn from(err: TDSTDError) -> Error {
         match err.kind() {
-            TDSTDErrorKind::FileExists => {
-                Error { kind: ErrorKind::FileExists }
+            TDSTDErrorKind::FileExists => Error {
+                kind: ErrorKind::FileExists,
             },
-            TDSTDErrorKind::DirectoryMissing => {
-                Error { kind: ErrorKind::DirectoryMissing }
+            TDSTDErrorKind::DirectoryMissing => Error {
+                kind: ErrorKind::DirectoryMissing,
             },
             TDSTDErrorKind::IO(_) => {
                 let err = err.into_inner_io().unwrap();
-                Error { kind: ErrorKind::IO(err) }
-            },
+                Error {
+                    kind: ErrorKind::IO(err),
+                }
+            }
             TDSTDErrorKind::Other(_) => {
                 let err = err.into_inner_other().unwrap();
-                Error { kind: ErrorKind::Other(err) }
+                Error {
+                    kind: ErrorKind::Other(err),
+                }
             }
         }
     }
@@ -75,7 +78,7 @@ impl From<&str> for Error {
     fn from(err: &str) -> Error {
         Error {
             kind: ErrorKind::Str(err.to_string()),
-	}
+        }
     }
 }
 
@@ -83,17 +86,15 @@ impl From<String> for Error {
     fn from(err: String) -> Error {
         Error {
             kind: ErrorKind::Str(err),
-	}
+        }
     }
 }
 
-
-
 impl From<ProtobufError> for Error {
     fn from(err: ProtobufError) -> Error {
         Error {
             kind: ErrorKind::Protobuf(err),
-	}
+        }
     }
 }
 
@@ -105,6 +106,7 @@ impl fmt::Display for Error {
             ErrorKind::FileExists => write!(f, "File already exists"),
             ErrorKind::InvalidApp => write!(f, "Invalid app response"),
             ErrorKind::DirectoryMissing => write!(f, "Destination path provided is not a valid directory"),
+            ErrorKind::SecurityCheck => write!(f, "Security check is needed, try to visit https://accounts.google.com/b/0/DisplayUnlockCaptcha to unlock, or setup an app-specific password"),
             ErrorKind::IO(err) => err.fmt(f),
             ErrorKind::Str(err) => err.fmt(f),
             ErrorKind::Protobuf(err) => err.fmt(f),
